<?php

namespace Ems\Services;

use Util\NovoCodigo;
use Util\Environment;
use Ems\Services\OperadorService;
use Ems\Repository\AvaliacaoRepository;
use Ems\Util\{ ExpectedException, Utilities };
use Teknisa\Model\Avaliacao\Entities\Questao;
use Teknisa\Model\Avaliacao\Entities\Resposta;
use Doctrine\ORM\EntityManager;

class AvaliacaoService {
    /** @var Environment */
    protected $environment;

    /** @var AvaliacaoRepository */
    protected $avaliacaoRepository;

    /** @var OperadorService */
    protected $operadorService;

    /** @var EntityManager */
    protected $entityManager;

    public function __construct(
        Environment         $environment,
        AvaliacaoRepository $avaliacaoRepository,
        OperadorService     $operadorService,
        EntityManager       $entityManager
    ) {
        $this->environment          = $environment;
        $this->avaliacaoRepository  = $avaliacaoRepository;
        $this->operadorService      = $operadorService;
        $this->entityManager        = $entityManager;
    }

    public function getAvaliacoes(
        string $cdOperador
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $operador = $this->operadorService->get($cdOperador);
        $nrParcNegocio = $operador['NRPARCNEGOCIO'];

        $avaliacoes = $this->avaliacaoRepository->getAvaliacoes(
            $nrOrg, $nrParcNegocio
        );

        return $avaliacoes;
    }

    public function getQuestoesAvaliacao(
        int $nrParticipAval
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $participAval = $this->getParcticipAval($nrParticipAval);

        $nrAvaliacao = (int) $participAval['NRAVALIACAO'];
        $nrParcExec = (int) $participAval['NRPARCEXECAVAL'];
        $nrParcAvaliado = (int) $participAval['NRPARCAVALIADO'];
        $nrCombQuest = (int) $participAval['NRCOMBQUESTAVAL'];

        $params = [
            'P_NRORG' => $nrOrg,
            'P_NRAVALIACAO' => $nrAvaliacao,
            'P_NRPARCEXECAVAL' => $nrParcExec,
            'P_NRPARCAVALIADO' => $nrParcAvaliado,
            'P_NRCOMBQUESTAVAL' => $nrCombQuest,
        ];

        $questoes = $this->avaliacaoRepository->getQuestoesAvaliacao(
            $params
        );

        foreach ($questoes as &$questao) {
            $nrQuestao = (int) $questao['NRQUESTAO'];
            $respostas = $this->avaliacaoRepository->getRespostasQuestao($nrOrg, $nrQuestao);

            $questao['respostas'] = $respostas;
        }

        return $questoes;
    }

    public function salvarAvaliacao(
        array $row
    ) {
        $this->entityManager->beginTransaction();

        try {
            $nrParticipAval = $row['NRPARTICIPAVAL'];
            $respostas = $row['respostas'];
            $cdOperador = $row['CDOPERADOR'];
            // flag que determina se é para concluir a avaliação
            $concluir = $row['concluir'];

            $participAval = $this->getParcticipAval($nrParticipAval);

            $nrAvaliacao = $participAval['NRAVALIACAO'];
            $nrParcAvaliado  = $participAval["NRPARCAVALIADO"];
            $nrParcExec = $participAval['NRPARCEXECAVAL'];
            $nrCombQuest = $participAval['NRCOMBQUESTAVAL'];

            foreach ($respostas as $nrQuestao => $resposta) {
                // $resposta pode ser string (resposta aberta) ou int (NRRESPOSTA da resposta)
                $dsResposta = $resposta;
                $nrResposta = null;

                if (is_int($resposta)) {
                    $respostaObj = $this->getResposta(
                        $resposta
                    );

                    $dsResposta = $respostaObj['DSRESPOSTA'];
                    $nrResposta = $resposta;
                }

                $execAvaliacao = $this->getExecAvaliacao(
                    $nrAvaliacao,
                    $nrParcExec,
                    $nrParcAvaliado,
                    $nrQuestao,
                    $nrCombQuest
                );

                $nrExecAvaliacao = $execAvaliacao['NREXECAVALIACAO'];

                $this->setRespAvaliacao(
                    $nrExecAvaliacao,
                    $nrResposta,
                    $dsResposta,
                    $cdOperador
                );
            }

            $this->updateExecParticipAvaliacao(
                $nrAvaliacao,
                $nrParcExec,
                $nrParcAvaliado,
                $nrCombQuest,
                $cdOperador
            );

            if ($concluir) {
                $this->concluirAvaliacao(
                    $nrAvaliacao,
                    $nrParcExec,
                    $nrParcAvaliado,
                    $nrCombQuest,
                    $cdOperador
                );
            }

            $this->entityManager->commit();
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }

    public function getResposta(
        int $nrResposta,
        $throwException = true
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();

        $resposta = $this->avaliacaoRepository->getResposta(
            $nrOrg, $nrResposta
        );

        if ($resposta === false && $throwException) {
            $message = "Não foi possível encontrar a resposta de NR $nrResposta!";
            throw new ExpectedException($message);
        }

        return $resposta;
    }

    public function getQuestao(
        int $nrQuestao,
        $throwException = true
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();

        $questao = $this->avaliacaoRepository->getQuestao(
            $nrOrg, $nrQuestao
        );

        if ($questao === false && $throwException) {
            $message = "Não foi possível encontrar a questão de NR $nrQuestao!";
            throw new ExpectedException($message);
        }

        return $questao;
    }

    public function getExecAvaliacao(
        int $nrAvaliacao,
        int $nrParcExec,
        int $nrParcAvaliado,
        int $nrQuestao,
        int $nrCombQuest
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();

        $params = [
            'P_NRORG' => $nrOrg,
            'P_NRAVALIACAO' => $nrAvaliacao,
            'P_NRPARCEXECAVAL' => $nrParcExec,
            'P_NRPARCAVALIADO' => $nrParcAvaliado,
            'P_NRQUESTAO' => $nrQuestao,
            'P_NRCOMBQUESTAVAL' => $nrCombQuest,
        ];

        $execAvaliacao = $this->avaliacaoRepository->getExecAvaliacao($params);

        if ($execAvaliacao === false) {
            $message = "Não foi possível encontrar a exec avaliação da questão de NR $nrQuestao!";
            throw new ExpectedException($message);
        }

        return $execAvaliacao;
    }

    public function setRespAvaliacao(
        int $nrExecAvaliacao,
        $nrResposta,
        $dsResposta,
        string $cdOperador
    ) {
        $respAvaliacao = $this->getRespAvaliacao($nrExecAvaliacao, false);

        if ($respAvaliacao === false) {
            $this->insertRespAvaliacao(
                $nrExecAvaliacao,
                $nrResposta,
                $dsResposta,
                $cdOperador
            );
        } else {
            $nrRespAvaliacao = $respAvaliacao['NRRESPAVALIACAO'];
            $this->updateRespAvaliacao(
                $nrRespAvaliacao,
                $nrResposta,
                $dsResposta,
                $cdOperador
            );
        }
    }

    public function getRespAvaliacao(
        int $nrExecAvaliacao,
        $throwException = true
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();

        $respAvaliacao = $this->avaliacaoRepository->getRespAvaliacao(
            $nrOrg, $nrExecAvaliacao
        );

        if ($respAvaliacao === false && $throwException) {
            $message = "Não foi possível encontrar a resposta da exec avaliação de NR $nrExecAvaliacao!";
            throw new ExpectedException($message);
        }

        return $respAvaliacao;
    }

    public function insertRespAvaliacao(
        int $nrExecAvaliacao,
        $nrResposta,
        $dsResposta,
        string $cdOperador
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();
        // commit = false pois há um $entity->commit() no final da função salvarAvaliacao
        $nrRespAvaliacao = NovoCodigo::geraCodigo($nrOrg, 'RESPAVALIACAO', 1, 10, true, false);

        $params = [
            'NRORG' => $nrOrg,
            'NRRESPOSTA' => $nrResposta,
            'DSRESPOSTA' => $dsResposta,
            'NREXECAVALIACAO' => $nrExecAvaliacao,
            'NRRESPAVALIACAO' => $nrRespAvaliacao,
            'CDOPERADOR' => $cdOperador,
        ];

        $this->avaliacaoRepository->insertRespAvaliacao($params);
    }

    public function updateRespAvaliacao(
        int $nrRespAvaliacao,
        $nrResposta,
        $dsResposta,
        $cdOperador
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();

        $params = [
            'P_NRORG' => $nrOrg,
            'P_NRRESPAVALIACAO' => $nrRespAvaliacao,
            'P_NRRESPOSTA' => $nrResposta,
            'P_DSRESPOSTA' => $dsResposta,
            'P_CDOPERADOR' => $cdOperador,
        ];

        $this->avaliacaoRepository->updateRespAvaliacao($params);
    }

    public function updateExecParticipAvaliacao(
        int $nrAvaliacao,
        int $nrParcExec,
        int $nrParcAvaliado,
        int $nrCombQuest,
        string $cdOperador
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();

        $params = [
            'P_NRORG' => $nrOrg,
            'P_NRPARCEXECAVAL' => $nrParcExec,
            'P_NRPARCAVALIADO' => $nrParcAvaliado,
            'P_NRAVALIACAO' => $nrAvaliacao,
            'P_CDOPERADOR' => $cdOperador,
            'P_NRCOMBQUESTAVAL' => $nrCombQuest,
        ];

        $this->avaliacaoRepository->updateExecAvaliacao($params);
        $this->avaliacaoRepository->updateParticipAval($params);
    }

    public function concluirAvaliacao(
        int $nrAvaliacao,
        int $nrParcExec,
        int $nrParcAvaliado,
        int $nrCombQuest,
        string $cdOperador
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();

        $params = [
            'P_NRORG' => $nrOrg,
            'P_NRPARCEXECAVAL' => $nrParcExec,
            'P_NRPARCAVALIADO' => $nrParcAvaliado,
            'P_NRAVALIACAO' => $nrAvaliacao,
            'P_CDOPERADOR' => $cdOperador,
            'P_NRCOMBQUESTAVAL' => $nrCombQuest,
        ];

        $this->avaliacaoRepository->concluirAvaliacao($params);
    }

    public function getParcticipAval(
        int $nrParticipAval,
        $throwException = true
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();
        $participAval = $this->avaliacaoRepository->getParticipAval($nrOrg, $nrParticipAval);

        if ($participAval === false && $throwException) {
            $message = "Não foi possível encontrar a participação de NR $nrParticipAval!";
            throw new ExpectedException($message);
        }

        return $participAval;
    }
}